
/*************************************************
****                   VEGA                   ****
****     Color managment for OpenGL V1.0      ****
**** Copyright 1996-2001, Alessandro Pedretti ****
*************************************************/


#ifdef __WIN32__
#  include <windows.h>
#endif

#include <GL\gl.h>
#include <stdio.h>

#include "globdef.h"
#include "globvar.h"

#define  __COLORTABLE
#include "gl_global.h"
#include "gl_colors.h"

#define  VG_DEFAULTCOL          VGCOL_MAGENTA


/**** Color a molecule by atom ****/

void GL_ColorAtm(register ATOMO *Atm)
{
  register VG_ATMCOL       *AtmCol;
  register VG_UBYTE        CurCol;

  while(Atm) {
    for(AtmCol = AtmColTbl; (AtmCol -> Elem.S) &&
       (Atm -> Elem.S != AtmCol -> Elem.S); ++AtmCol);
    Atm -> ColorID = AtmCol -> ColorID;
    Atm = Atm -> Ptr;
  }
}


/**** Color by molecule ****/

void GL_ColorByMol(register ATOMO *Atm)
{
  VG_UBYTE      ColorID = VGCOL_WHITE;

  while(Atm) {
    Atm -> ColorID = ColorID;
    if (Atm -> Flags & VG_ATMF_SEGEND) {
      ++ColorID;
      if (ColorID > VGCOL_NUM) ColorID = VGCOL_WHITE;
    }
    Atm            = Atm -> Ptr;
  }
}


/**** Color a molecule ****/

void GL_ColorMol(register ATOMO *Atm, register VG_UBYTE ColorID)
{
  if (ColorID > VGCOL_NUM) ColorID = VGCOL_NUM;

  while(Atm) {
    Atm -> ColorID = ColorID;
    Atm = Atm -> Ptr;
  }
}


/**** Color surface ****/

void GL_ColorSrf(register VG_SURFACE *Dot, VG_UBYTE ColorID)
{
  register VG_COLOR     *Col;

  if (ColorID > VGCOL_NUM) ColorID = VGCOL_NUM;
  Col = &ColorTable[ColorID];

  while(Dot) {
    Dot -> Color[0] = Col -> R;
    Dot -> Color[1] = Col -> G;
    Dot -> Color[2] = Col -> B;
    Dot = Dot -> Ptr;
  }
}

/**** Color surface by atom ****/

void GL_ColorSrfAtm(register VG_SURFACE *Dot, register ATOMO *Atm)
{
  register VG_ATMCOL    *AtmCol;
  VG_UBYTE              R, G, B;

  register VG_ULONG     PrecAtmNum = 0;

  while(Dot) {
    if (Dot -> AtmNum != PrecAtmNum) {

      /**** Search the atom ****/

      while((Dot -> AtmNum != Atm -> Num) && (Atm))
        Atm = Atm -> Ptr;

      /**** Search the color ****/

      for(AtmCol = AtmColTbl; (AtmCol -> Elem.S) &&
       (Atm -> Elem.S != AtmCol -> Elem.S); ++AtmCol);

      R = ColorTable[AtmCol -> ColorID].R;
      G = ColorTable[AtmCol -> ColorID].G;
      B = ColorTable[AtmCol -> ColorID].B;
    }
    Dot -> Color[0] = R;
    Dot -> Color[1] = G;
    Dot -> Color[2] = B;
    PrecAtmNum = Dot -> AtmNum;
    Dot = Dot -> Ptr;
  } /* End of dot loop */
}


/**** Color surface by color ****/

void GL_ColorSrfCol(register VG_SURFACE *Dot, VG_ULONG Color)
{
  VG_UBYTE      R = Color & 0xff;
  VG_UBYTE      G = (Color & 0xff00) / 256;
  VG_UBYTE      B = (Color & 0xff0000) / 65536;

  while(Dot) {
    Dot -> Color[0] = R;
    Dot -> Color[1] = G;
    Dot -> Color[2] = B;
    Dot = Dot -> Ptr;
  } /* End of while */
}


/**** Color surface by values ****/

void GL_ColorSrfVal(VG_SURFACE *InizSrf)
{
  float       Step, Val1x, Val2x, Val3x;
  register VG_SURFACE   *Dot;

  float        MaxVal = InizSrf -> Val;
  float        MinVal = InizSrf -> Val;

  /**** Find max and min values ****/

  for(Dot = InizSrf -> Ptr;Dot;Dot = Dot -> Ptr) {
    if (Dot -> Val > MaxVal) MaxVal = Dot -> Val;
    if (Dot -> Val < MinVal) MinVal = Dot -> Val;
  }

  Step = (MaxVal - MinVal) / 4.0;
  if (Step == 0.0) Step = 1.0; /* To avoid division by zero */

  Val1x = MinVal + Step;
  Val2x = Val1x  + Step;
  Val3x = Val2x  + Step;

  /**** Set the color of each dot ****/

  for(Dot = InizSrf -> Ptr;Dot;Dot = Dot -> Ptr) {
    if (Dot -> Val < Val1x) {
      Dot -> Color[0] = 0;
      Dot -> Color[1] = (VG_UBYTE)(((Dot -> Val - Val1x) / Step) * 255.0);
      Dot -> Color[2] = 255;
    } else if (Dot -> Val < Val2x) {
      Dot -> Color[0] = 0;
      Dot -> Color[1] = 255;
      Dot -> Color[2] = (VG_UBYTE)(((Val2x - Dot -> Val) / Step) * 255.0);
    } else if (Dot -> Val < Val3x) {
      Dot -> Color[0] = (VG_UBYTE)(((Dot -> Val - Val2x) / Step) * 255.0);
      Dot -> Color[1] = 255;
      Dot -> Color[2] = 0;
    } else {
      Dot -> Color[0] = 255;
      Dot -> Color[1] = (VG_UBYTE)(((MaxVal - Dot -> Val) / Step) * 255.0);
      Dot -> Color[2] = 0;
    }
  } /* End of dot for */
}


