
/*************************************************
****                   VEGA                   ****
****      Routines for OpenGL Demo V1.0       ****
**** Copyright 1996-2001, Alessandro Pedretti ****
*************************************************/


#ifdef __WIN32__
#  include <windows.h>
#endif

#include <GL\gl.h>
#include <stdio.h>

#include "globdef.h"
#include "globvar.h"
#include "globstr.h"
#include "gl_global.h"
#include "gl_colors.h"
#include "gl_demo.h"

#ifdef WIN32
#  include "Win32\winutils.h"
#endif

#include "gl_dmprg.h"


/**** Local variables ****/

static DEMO_EVENT    *PC;
static float         DemoCnt;

/**** Local prototypes ****/

static VG_BOOL GL_DemoGoto(void);


/**** Callback function ****/

VG_BOOL GL_DemoCallBack(VG_GLPREFS *Prf, float TS, VG_BOOL *Force)
{
  char                  *Str;
  GLYPHMETRICS          Metrics;
  register ATOMO        *Atm;
  VG_ULONG              Len;

  VG_BOOL               Ret = TRUE;
  VG_BOOL               Brk = FALSE;

  while ((Ret) && (!Brk)) {
    switch(PC -> Event) {

    case DEMO_COM_ATMLBL:
      Prf -> AtmLbl = PC -> Par.I;
      break;

    case DEMO_COM_CENTX:
      Prf -> Center.x = PC -> Par.F;
      break;

    case DEMO_COM_CENTY:
      Prf -> Center.y = PC -> Par.F;
      break;

    case DEMO_COM_CENTZ:
      Prf -> Center.z = PC -> Par.F;
      break;

    case DEMO_COM_COLAT:
      GL_ColorAtm(BegAtm);
      break;

    case DEMO_COM_COLOR:
      GL_ColorMol(BegAtm, PC -> Par.I);
      break;

    case DEMO_COM_DRAW:
      Prf -> DrawMode = PC -> Par.I;
      break;

    case DEMO_COM_EXPLODE:
      DemoCnt += TS;
      if (DemoCnt < (float)(PC -> Par.I)) {
        for(Atm = BegAtm;Atm; Atm = Atm -> Ptr) {
          Atm -> x *= PC -> Par.F;
          Atm -> y *= PC -> Par.F;
          Atm -> z *= PC -> Par.F;
        }
        Brk = TRUE;
        --PC;
      } else DemoCnt = 0;
      break;

    case DEMO_COM_FX:
      Prf -> DemoFX = !Prf -> DemoFX;
      if (Prf -> DemoFX)
        glEnable(GL_TEXTURE_2D);
      else
        glDisable(GL_TEXTURE_2D);
      break;

    case DEMO_COM_GOTO:
      GL_DemoGoto();
      break;

    case DEMO_COM_IF:
      if (PC -> Par.F == DMFL_NOTSUBTITLES) {
        if (!Prf -> DemoText) GL_DemoGoto();
      } /* End of switch */
      break;

    case DEMO_COM_LOAD:
      if (BegAtm = Loader(GL_DemoGetPath(PC -> Par.C), &TotalAtm, 1)) {
        if (BegAtm != (ATOMO *)-2) {
          Connect(BegAtm, TotalAtm);
          AssignVdwNew(BegAtm, 0, 0);
          *Force          = TRUE;
        } else BegAtm = NULL;
      } else Ret = FALSE;
      break;

    case DEMO_COM_NEW:
      GL_Reset(Prf);
      Prf -> Animation = TRUE;
      Prf -> DemoFX    = FALSE;
      break;

    case DEMO_COM_NORM:
      Normalize(BegAtm, TotalAtm);
      break;

    case DEMO_COM_PLAYMIDI:
#ifdef WIN32
      if (ViewPrefs.PlayMusic)
        PlayMidi(GL_DemoGetPath(PC -> Par.C));
#endif
      break;

    case DEMO_COM_ROTX:
      Prf -> StepRot.x = PC -> Par.F;
      break;

    case DEMO_COM_ROTY:
      Prf -> StepRot.y = PC -> Par.F;
      break;

    case DEMO_COM_ROTZ:
      Prf -> StepRot.z = PC -> Par.F;
      break;

    case DEMO_COM_SKIP:
      PC += PC -> Par.I;
      break;

    case DEMO_COM_SLEEP:
      DemoCnt += TS;
      if (DemoCnt < (float)(PC -> Par.I)) {
        Brk = TRUE;
        --PC;
      } else DemoCnt = 0;
      break;

    case DEMO_COM_SOLVS:
      Prefs.SOL_SHAPE = SOL_SHAPE_SPHERE;
      Prefs.SOL_RAD   = PC -> Par.F;
      Ret = Solvate(GL_DemoGetPath(".." C_DIRSEP "Data" C_DIRSEP "Clusters"
                    C_DIRSEP "Water"), BegAtm, &TotalAtm);
      break;

    case DEMO_COM_STEPZ:
      Prf -> StepTrans.z = PC -> Par.F;
      break;

    case DEMO_COM_STOPMIDI:
#ifdef WIN32
      StopMidi();
#endif
      break;

    case DEMO_COM_SURFCALC:
      BegSrf = SrfCalc(&TotalSrf, BegAtm, TotalAtm, PC -> Par.I, 0.0, SRF_PROP_MEP, NULL);
      ViewPrefs.SrfAct = TRUE;
      break;

    case DEMO_COM_SURFCLR:
      CloseSrf();
      break;

    case DEMO_COM_SURFCOLAT:
      GL_ColorSrfAtm(BegSrf, BegAtm);
      break;

    case DEMO_COM_SURFCOLP:
      GL_ColorSrfVal(BegSrf);
      break;

    case DEMO_COM_TEXT:
      if (*PC -> Par.C) {
        glNewList(Prf -> TxtList, GL_COMPILE);
          glPushMatrix();
            glTranslatef(Prf -> Center.x, Prf -> Center.y, -20);
            glColor3ub(255, 255, 255);
/*
            if (PC -> Par.I == TXT_CENTER) {
              Len = 0;
              for(Str = PC -> Par.C; *Str; ++Str) {
                GetGlyphOutline(hDC, *Str, GGO_METRICS, &Metrics, sizeof(GLYPHMETRICS),
                                NULL, NULL);
		Len += Metrics.gmCellIncX;
              }
            //  glRasterPos2f((float)Len * -0.00028, 0.0);
            //    glRasterPos2f(-(float)(Prf -> SsizeX - Prf -> WsizeX) * 0.005 -(float)Len * -0.00028 , 0.0);
                glRasterPos2f(1 - (float)Prf -> WsizeX / (float)Prf -> WsizeY -(float)Len * 0.0003, 0.0);
            } else glRasterPos2f(TextPos[PC -> Par.I].x * (float)Prf -> WsizeX / (float)Prf -> WsizeY , TextPos[PC -> Par.I].y);
*/
            glRasterPos2f(TextPos[PC -> Par.I].x * (float)Prf -> WsizeX / (float)Prf -> WsizeY , TextPos[PC -> Par.I].y);
            GL_Print(PC -> Par.C);
          glPopMatrix();
        glEndList();
        Prf -> TxtAct = TRUE;
      } else Prf -> TxtAct = FALSE;
      break;

    } /* End of switch */

    ++PC;
    if (PC -> Event == DEMO_COM_LOOP) {
      PC = EventList;
      break;
    }
  }

  if (!Ret) {
    GL_DemoStop(Prf);
    Errore("Demo error");
  }

  return Ret;
}


/**** Goto function ****/

static VG_BOOL GL_DemoGoto(void)
{
  register DEMO_EVENT    *NewPC = EventList;
  VG_BOOL                Ret    = TRUE;

  while(Ret) {
    if (NewPC -> Event != DEMO_COM_END) {
      if ((NewPC -> Event == DEMO_COM_LABEL) && (PC -> Par.I == NewPC -> Par.I)) {
        PC = NewPC;
        break;
      }
      ++NewPC;
    } else Ret = FALSE;
  } /* End of for */

  return Ret;
}


/**** Initialize the OpenGL demo ****/

void GL_DemoStart(VG_GLPREFS *Prf)
{
  GL_Reset(Prf);
  GL_DemoInitFX();

  DemoCnt          = 0.0;
  PC               = EventList;
  Prf -> Animation = TRUE;
  Prf -> Demo      = TRUE;
  Prf -> TxtList   = glGenLists(1);
  GL_DrawScene(Prf, TRUE);
  GL_KillFont();
  GL_BuildFont(-12, 8);
}


/**** Stop the OpenGL demo ****/

void GL_DemoStop(VG_GLPREFS *Prf)
{
#ifdef WIN32
  StopMidi();
#endif
  Prf -> Demo = FALSE;
  if (Prf -> DemoFX) {
    glDisable(GL_TEXTURE_2D);
    Prf -> DemoFX = FALSE;
  }

  GL_Reset(Prf);
  GL_DemoRelaseFX();
}


/**** Get the demo directory with path ****/

char *GL_DemoGetPath(char *File)
{
  static char         Buf[512];

  GetPrgPath(Buf, FALSE);
  strcat(Buf, "Demo\\");
  strcat(Buf, File);

  return &Buf[0];
}

