
/*************************************************
****                   VEGA                   ****
****        OpenGL Global Header V1.0         ****
**** Copyright 1996-2001, Alessandro Pedretti ****
*************************************************/


#ifndef  __GL_GLOBAL_H
#define  __GL_GLOBAL_H

/**** Constants ****/

#define  VG_GLDEF_MANROTSTEP            0.5
#define  VG_GLDEF_ROTSTEP               0.3
#define  VG_GLDEF_ROTSTEPINC            0.1

/**** Draw modes ****/

#define  VG_GLDRAW_WIREFRAME            1
#define  VG_GLDRAW_VDW_WIRE             2
#define  VG_GLDRAW_VDW_SOLID            3
#define  VG_GLDRAW_CPK_WIRE             4
#define  VG_GLDRAW_CPK_SOLID            5
#define  VG_GLDRAW_MODES                5

/**** Mouse modes ****/

#define  VG_MOUSE_ROT                   0
#define  VG_MOUSE_TRANS                 1
#define  VG_MOUSE_SCALE                 2
#define  VG_MOUSE_MODES                 3


/**** Structures ****/

typedef struct {
  XYZ           Center;                 /* View center           */
  XYZ           Rot;                    /* Rotation              */
  XYZ           StepRot;                /* Steps for rotation    */
  XYZ           StepTrans;              /* Steps for translation */
  float         Scale;                  /* Scale                 */
  VG_ULONG      WposX, WposY;           /* Window position       */
  VG_ULONG      WsizeX, WsizeY;         /* Window size           */
  VG_ULONG      Wdepth;                 /* Window bitplanes      */
  VG_ULONG      SsizeX, SsizeY;         /* Screen size           */
  VG_ULONG      Sdepth;                 /* Screen bitplanes      */
  VG_BOOL       FullScreen;             /* Full screen flag      */
  VG_BOOL       Animation;              /* Animation mode        */
  VG_BOOL       Demo;                   /* Demo mode             */
  VG_BOOL       DemoText;               /* Show text in demo     */
  VG_UWORD      DrawMode;               /* Draw mode             */
  VG_ULONG      Refresh;                /* Refresh time          */
  VG_ULONG      TotMol;                 /* Loaded molecules      */
  VG_BOOL       ShowFps;                /* Show fps              */
  VG_BOOL       PlayMusic;              /* Play the music        */
  VG_ULONG      TxtList;                /* Text list             */
  VG_BOOL       TxtAct;                 /* Text list active      */
  VG_BOOL       SrfAct;                 /* Surface list active   */
  VG_BOOL       Light;                  /* Light                 */
  VG_UWORD      MouseMode;              /* Mouse mode            */
  DWORD         AtmLbl;                 /* Atom label            */
  VG_ULONG      DemoFX;                 /* Demo FX active        */
} VG_GLPREFS;

/**** Default preferences ****/

#ifdef __OPENGL_MAIN
VG_GLPREFS  ViewPrefs  = { {0.0, 0.0, 0.0},     /* View center           */
                           {0.0, 0.0, 0.0},     /* Rotation              */
                           {0.0, 0.0, 0.0},     /* Steps for rotation    */
                           {0.0, 0.0, 0.0},     /* Step for translation  */
                           1.0,                 /* Scale                 */
                           0, 0,                /* Window position       */
                           640, 480,            /* Window size           */
                           16,                  /* Window bitplanes      */
                           640, 480,            /* Screen size           */
                           32,                  /* Screen bitplanes      */
                           FALSE,               /* Full screen flag      */
                           FALSE,               /* Animation mode        */
                           FALSE,               /* Demo mode             */
                           TRUE,                /* Show text in demo     */
                           VG_GLDRAW_WIREFRAME, /* Draw mode             */
                           1000 / 75,           /* Refresh time          */
                           0,                   /* Loaded molecules      */
                           FALSE,               /* Show fps              */
                           TRUE,                /* Play the music        */
                           0,                   /* Text list             */
                           FALSE,               /* Text list active      */
                           FALSE,               /* Surface list active   */
                           TRUE,                /* Light                 */
                           VG_MOUSE_ROT,        /* Mouse mode            */
                           VG_MM_LBLATMOFF,     /* Atom label            */
                           0                    /* Demo FX active        */
                         };
#  ifdef WIN32
HDC     hDC  = NULL;
HWND    hWnd = NULL;
#  endif
#else
extern VG_GLPREFS  ViewPrefs;

#  ifdef WIN32
extern HDC     hDC;
extern HWND    hWnd;
#  endif
#endif

/**** Prototipes ****/

#  ifdef __cplusplus
extern "C" {
#  endif
void     ChangeAnimMenu(VG_BOOL);
void     ChangeLblAtmMenu(VG_ULONG);
void     ChangeLightMenu(VG_BOOL);
void     ChangeMousePopup(void);
HWND     CreateGLWindow(char *, VG_GLPREFS *);
VG_BOOL  GL_BuildFont(int, int);
void     GL_ChangeMenu(VG_ULONG);
void     GL_ColorAtm(ATOMO *);
void     GL_ColorByMol(ATOMO *);
void     GL_ColorMol(ATOMO *, VG_UBYTE);
void     GL_ColorSrf(VG_SURFACE *, VG_UBYTE);
void     GL_ColorSrfAtm(VG_SURFACE *, ATOMO *);
void     GL_ColorSrfCol(VG_SURFACE *, VG_ULONG);
void     GL_ColorSrfVal(VG_SURFACE *);
VG_BOOL  GL_DemoCallBack(VG_GLPREFS *, float, VG_BOOL *);
void     GL_DemoDrawFX(void);
char     *GL_DemoGetPath(char *);
VG_BOOL  GL_DemoInitFX(void);
void     GL_DemoRelaseFX(void);
void     GL_DemoStart(VG_GLPREFS *);
void     GL_DemoStop(VG_GLPREFS *);
void     GL_DrawAtmLbl(ATOMO *);
void     GL_DrawCpk(ATOMO *, VG_BOOL);
void     GL_DrawDev(VG_GLPREFS *);
VG_BOOL  GL_DrawScene(VG_GLPREFS *, VG_BOOL);
void     GL_DrawWireframe(ATOMO *);
void     GL_DrawVdw(ATOMO *, VG_BOOL);
void     GL_KillFont(void);
void     GL_Print(const char *, ...);
VG_BOOL  GL_PrintWindow(void);
void     GL_Reset(VG_GLPREFS *);
void     GL_ResetView(VG_GLPREFS *);
VG_BOOL  GL_SaveWindow(char *);
void     GL_ShowSrf(VG_SURFACE *);
VG_BOOL  InitGL(void);
void     InitMoPac(ATOMO *, VG_ULONG);
void     KillGLWindow(HWND);
void     ReSizeGLScene(VG_ULONG, VG_ULONG);

#  ifdef __cplusplus
}
#  endif
#endif

