
/*************************************************
****                   VEGA                   ****
****          OpenGL Utilities V1.0           ****
**** Copyright 1996-2001, Alessandro Pedretti ****
*************************************************/


#ifdef __WIN32__
#  include <windows.h>
#endif

#include <GL\gl.h>
#include <GL\glu.h>
#include <GL\glaux.h>
#include <stdio.h>

#include "globdef.h"
#include "globvar.h"
#include "gl_global.h"
#include "gl_colors.h"
#include "gl_menu.h"

#define  CPK_VDW_SCALE          0.2
#define  CPK_LINE_WIDTH         3.0

static GLuint	WireList = 0;


/**** Draw atom labels ****/

void GL_DrawAtmLbl(register ATOMO *Atm)
{
  while(Atm) {
//    glPushMatrix();
    if ((ViewPrefs.AtmLbl == VG_MM_LBLATMTYP) && (*Atm -> Pot.C == '?'))
      GL_SetColor(VGCOL_RED);
    else
      GL_SetColor(VGCOL_WHITE);

    glRasterPos3f(Atm -> x, Atm -> y, Atm -> z);

    switch(ViewPrefs.AtmLbl) {
    case VG_MM_LBLATMNAM:       /* Atom name */
      GL_Print("%.4s", Atm -> Name.C);
      break;
    case VG_MM_LBLATMELM:       /* Element name */
      GL_Print("%.4s", Atm -> Elem.C);
      break;
    case VG_MM_LBLATMNUM:       /* Atom number */
      GL_Print("%d", Atm -> Num);
      break;
    case VG_MM_LBLATMTYP:       /* Atom type */
      GL_Print("%.4s", Atm -> Pot.C);
      break;
    case VG_MM_LBLATMCHG:       /* Atom charge */
      GL_Print("%.2f", Atm -> Charge);
      break;
    } /* End of switch */
//    glPopMatrix();
    Atm = Atm -> Ptr;
  } /* End of while */
}


/**** Draw CPK ****/

void GL_DrawCpk(register ATOMO *Atm, VG_BOOL Wire)
{
  register ATOMO        **ConnAtm;
  register VG_ULONG     k;


  while(Atm) {
  glLineWidth(1.0);
    glPushMatrix();
      GL_SetColor(Atm -> ColorID);
      glTranslatef(Atm -> x, Atm -> y, Atm -> z);
      if (Wire)
        auxWireSphere(Atm -> Rad * CPK_VDW_SCALE);
      else
        auxSolidSphere(Atm -> Rad * CPK_VDW_SCALE);
    glPopMatrix();

    if (Atm -> NSost) {
      glLineWidth(CPK_LINE_WIDTH);
      glBegin(GL_LINES);
        ConnAtm = Atm -> Conn;
        for(k = 0; k < (VG_ULONG)Atm -> NSost; ++k) {
          if ((*ConnAtm) -> Num > Atm -> Num) {
            GL_SetColor(Atm -> ColorID);
            glVertex3f(Atm -> x, Atm -> y, Atm -> z);
            GL_SetColor((*ConnAtm) -> ColorID);
            glVertex3f((*ConnAtm) -> x, (*ConnAtm) -> y, (*ConnAtm) -> z);
          }
          ++ConnAtm;
        }
      glEnd();
    }
    Atm = Atm -> Ptr;
  } /* End of while */

/*
    if (Wire) auxWireCylinder(0.5, 1.0);
    else auxSolidCylinder(0.5, 1.0);
*/

}


/**** Draw in wireframe mode ****/

void GL_DrawWireframe(register ATOMO *Atm)
{
  register ATOMO        **ConnAtm;
  register VG_ULONG     k;

  glLineWidth(1.0);
  while(Atm) {
    if (Atm -> NSost) {
      glBegin(GL_LINES);
        ConnAtm = Atm -> Conn;
        for(k = 0; k < (VG_ULONG)Atm -> NSost; ++k) {
          if ((*ConnAtm) -> Num > Atm -> Num) {
            GL_SetColor(Atm -> ColorID);
            glVertex3f(Atm -> x, Atm -> y, Atm -> z);
            GL_SetColor((*ConnAtm) -> ColorID);
            glVertex3f((*ConnAtm) -> x, (*ConnAtm) -> y, (*ConnAtm) -> z);
          }
          ++ConnAtm;
        }
      glEnd();
    } else {
      GL_SetColor(Atm -> ColorID);
      glBegin(GL_POINTS);
      glVertex3f(Atm -> x, Atm -> y, Atm -> z);
      glEnd();
    }
    Atm = Atm -> Ptr;
  } /* End of while */
}

/**** Draw Van der Waals spheres ****/

void GL_DrawVdw(register ATOMO *Atm, VG_BOOL Wire)
{
  glLineWidth(1.0);

  while(Atm) {
    glPushMatrix();
    GL_SetColor(Atm -> ColorID);
    glTranslatef(Atm -> x, Atm -> y, Atm -> z);
    if (Wire) auxWireSphere(Atm -> Rad);
    else auxSolidSphere(Atm -> Rad);
    glPopMatrix();
    Atm = Atm -> Ptr;
  }
  glFlush();
}


/**** Reset all memory data ****/

void GL_Reset(VG_GLPREFS *Prf)
{
  CloseAllMol();                         /* Free the memory          */
  Prf -> DrawMode    = VG_GLDRAW_WIREFRAME;
  Prf -> Rot.x       = 0.0;              /* Reset rotation matrix    */
  Prf -> Rot.y       = 0.0;
  Prf -> Rot.z       = 0.0;
  Prf -> StepRot.x   = 0.0;              /* Reset animation steps    */
  Prf -> StepRot.y   = VG_GLDEF_ROTSTEP;
  Prf -> StepRot.z   = 0.0;
  Prf -> StepTrans.x = 0.0;              /* Reset translations steps */
  Prf -> StepTrans.y = 0.0;
  Prf -> StepTrans.z = 0.0;
  Prf -> Center.x    = 0.0;              /* Reset rotation center    */
  Prf -> Center.y    = 0.0;
  Prf -> Center.z    = 0.0;
  Prf -> Animation   = FALSE;            /* Stop animation           */
  Prf -> Light       = TRUE;             /* Activate the light       */
  Prf -> TotMol      = 0;
  Prf -> TxtAct      = FALSE;            /* Disable text             */
  Prf -> SrfAct      = FALSE;            /* Disable surface          */
  Prf -> MouseMode   = VG_MOUSE_ROT;     /* Mouse mode rotation      */
  Prf -> AtmLbl      = VG_MM_LBLATMOFF;  /* Atom labels off          */

  ChangeLblAtmMenu(VG_MM_LBLATMOFF);
  ChangeMousePopup();
}


/**** Reset the view ****/

void GL_ResetView(VG_GLPREFS *Prf)
{
  Prf -> Rot.x       = 0.0;              /* Reset rotation matrix    */
  Prf -> Rot.y       = 0.0;
  Prf -> Rot.z       = 0.0;
  Prf -> StepRot.x   = 0.0;              /* Reset animation steps    */
  Prf -> StepRot.y   = VG_GLDEF_ROTSTEP;
  Prf -> StepRot.z   = 0.0;
  Prf -> StepTrans.x = 0.0;              /* Reset translations steps */
  Prf -> StepTrans.y = 0.0;
  Prf -> StepTrans.z = 0.0;
  Prf -> Animation   = FALSE;            /* Stop animation           */
  Prf -> Light       = TRUE;             /* Activate the light       */

  ChangeAnimMenu(FALSE);
  ChangeLightMenu(TRUE);
}

