
/*************************************************
****             VEGA - Locale V1.1           ****
****           Localization routines          ****
**** Copyright 1996-2001, Alessandro Pedretti ****
*************************************************/

/*
 * The following routines use the AmigaOS Locale Technology to translate the
 * strings in any language using a specific catalog file in standard IFF format.
 * Each locale file must be stored in a sub-directory with the lower-case language
 * name of the Locale directory and must have VEGA.catalog file name.
 * To create a new catalog, you must use FlexCat compiler with the catalog
 * description file (.cd) and the catalog translation file (.ct). Please read the
 * specific documentation in FlexCat directory and in VEGA manual.
 * The AmigaOS Locale Technology is copyrighted by Amiga Inc.
 */


#ifdef __WIN32__
#  include <windows.h>
#endif

#include <stdio.h>
#include <string.h>
#include <stdarg.h>

#include "globdef.h"
#include "globvar.h"
#define  VEGA_ARRAY
#include "globstr.h"
#include "../LocaleLib/locale.h"


#ifdef AMIGA
#  include <exec/libraries.h>
#  include <libraries/locale.h>

extern struct Library  *LocaleBase;
#endif

#ifdef WIN32
   extern HWND    hWnd;
#  define  ERROR_PROMPT "\n%s %s.\n\n"
#else
#  define  ERROR_PROMPT "\n\033[33m%s\033[0m %s.\n\n"
#endif


/**** Shows an error ****/

VG_BOOL CatErr(VG_ULONG Err, ...)
{
  char          Buf[256];
  va_list       vl;

#ifdef WIN32
  if (GLOBSW_OPENGL) {
    va_start(vl, Err);
    vsprintf(Buf, GetStr(Err), vl);
    va_end(vl);
    MessageBox(hWnd, Buf, GetStr(MSG_ERR_LOCALE_ERROR), MB_OK|MB_ICONERROR);
  } else {
#endif

    sprintf(Buf, ERROR_PROMPT, GetStr(MSG_ERR_LOCALE_ERROR), GetStr(Err));

#ifdef WIN32
    LocIso2Dos(Buf, FALSE);
#endif

    va_start(vl, Err);
    vfprintf(stderr, Buf, vl);
    va_end(vl);

#ifdef WIN32
  }
#endif

  return FALSE;
}


/**** fprintf replacement with string localization ****/

int CatPrintf(FILE *FH, VG_ULONG Str, ...)
{
#ifdef WIN32
  char          Buf[4096];
#endif

  int           Ret;
  va_list       vl;

  va_start(vl, Str);

#ifdef WIN32
  vsprintf(Buf, GetStr(Str), vl);
  LocIso2Dos(Buf, FALSE);
  Ret = fprintf(FH, Buf, vl);
#else
  Ret = vfprintf(FH, GetStr(Str), vl);
#endif
  va_end(vl);

  return Ret;
}


/**** Return a string and if possible, translate it ****/

char *GetStr(VG_ULONG Id)
{
  char                  *BuiltIn = "Unknown localized string";
  register VG_ULONG     k        = sizeof(VEGA_Str) / sizeof(LOCARRAY);
  register LOCARRAY     *Ptr     = (LOCARRAY *)VEGA_Str;

  while(k--) {
    if (Ptr -> Id == Id) {
      BuiltIn = Ptr -> Str;
      break;
    }
    ++Ptr;
  }

  /**** String localization ****/

  if (Catalog)
#ifdef AMIGA
    return((char *)GetCatalogStr((void *)Catalog, Id, BuiltIn));
#else
    return((char *)LocGetCatStr((void *)Catalog, Id, BuiltIn));
#endif

  return BuiltIn;
}


