
/*************************************************
****             VEGA - Bzip2 V1.1            ****
****            Interface to libbz2           ****
**** Copyright 1996-2000, Alessandro Pedretti ****
*************************************************/


#include <stdio.h>
#include <string.h>

#include "globstr.h"
#include "../Bzip2/bzlib.h"

#define  FALSE                  0
#define  TRUE                   1
#define  BZ2_BUFFER_SIZE        65536


/**** Local prototypes ****/

/* Some VEGA definitions are incompatible with the standard windows.h */
/* file and thus can't be used in the following prototypes            */

void         *Alloca(unsigned int);
short        CatErr(unsigned int, ...);
short        BZ2_DepackFile(char *, char *);
static short BZ2_Error(BZFILE *FH);
short        BZ2_PackFile(char *, char *);
short        PrintDosErr(void);


/**** Decompress a file into another ****/

short BZ2_DepackFile(char *Src, char *Dest)
{
  char          *Buf;
  FILE          *FO;
  int           Rd;

  BZFILE       *FI = NULL;
  short        Ret = TRUE;

  if ((Buf = (char *)Alloca(BZ2_BUFFER_SIZE))) {
    if ((FI = BZ2_bzopen(Src, "rb"))) {
      if ((FO = fopen(Dest, "wb"))) {
        while((Ret) && (Rd = BZ2_bzread(FI, Buf, BZ2_BUFFER_SIZE))) {
          if (Rd == -1)
            Ret = BZ2_Error(FI);
          else if (fwrite(Buf, Rd, 1, FO) != 1)
           Ret = PrintDosErr();
        }
        fclose(FO);
      } else Ret = PrintDosErr();
      BZ2_bzclose(FI);
    } else Ret = FALSE;
  } else Ret = BZ2_Error(FI);

  return Ret;
}


/**** Compress a file into another ****/

short BZ2_PackFile(char *Src, char *Dest)
{
  BZFILE        *FO;
  char          *Buf;
  FILE          *FI;
  int           Rd;

  short        Ret = TRUE;

  if ((Buf = (char *)Alloca(BZ2_BUFFER_SIZE))) {
    if ((FI = fopen(Src, "rb"))) {
      if ((FO = BZ2_bzopen(Dest, "wb"))) {
        while ((Ret) && (Rd = fread(Buf, 1, BZ2_BUFFER_SIZE, FI))) {
          if (BZ2_bzwrite(FO, Buf, Rd) != Rd)
            Ret = BZ2_Error(FO);
        }
        if (ferror(FI)) Ret = PrintDosErr();
        BZ2_bzclose(FO);
      } else Ret = BZ2_Error(FO);
      fclose(FI);
    } else Ret = PrintDosErr();
  } else Ret = FALSE;

  return Ret;
}


/**** Show a bzip2 error ****/

static short BZ2_Error(BZFILE *FH)
{
  int           err;

  return CatErr(MSG_ERR_BZIP2_ERROR, BZ2_bzerror(FH, &err));
}
