
/*************************************************
****              VEGA - GZip V1.1            ****
****             Interface to zlib            ****
**** Copyright 1996-2000, Alessandro Pedretti ****
*************************************************/


#include <stdio.h>
#include <string.h>

#include "globstr.h"
#include "../Zlib/zlib.h"

#define  FALSE                  0
#define  TRUE                   1
#define  GZ_BUFFER_SIZE         65536


/**** Local prototypes ****/

/* Some VEGA definitions are incompatible with the standard windows.h */
/* file and thus can't be used in the following prototypes            */

void         *Alloca(unsigned int);
short        CatErr(unsigned int, ...);
short        GZ_DepackFile(char *, char *);
static short GZ_Error(gzFile *FH);
short        GZ_PackFile(char *, char *);
short        PrintDosErr(void);


/**** Decompress a file into another ****/

short GZ_DepackFile(char *Src, char *Dest)
{
  char          *Buf;
  FILE          *FO;
  gzFile        FI;
  int           Rd;

  short        Ret = TRUE;

  if ((Buf = (char *)Alloca(GZ_BUFFER_SIZE))) {
    if ((FI = gzopen(Src, "rb"))) {
      if ((FO = fopen(Dest, "wb"))) {
        while((Ret) && (Rd = gzread(FI, Buf, GZ_BUFFER_SIZE))) {
          if (Rd == -1)
            Ret = GZ_Error(&FI);
          else if (fwrite(Buf, Rd, 1, FO) != 1)
           Ret = PrintDosErr();
        }
        fclose(FO);
      } else Ret = PrintDosErr();
      gzclose(FI);
    } else Ret = FALSE;
  } else Ret = GZ_Error(&FI);

  return Ret;
}


/**** Compress a file into another ****/

short GZ_PackFile(char *Src, char *Dest)
{
  char          *Buf;
  gzFile        FO;
  FILE          *FI;
  int           Rd;

  short        Ret = TRUE;

  if ((Buf = (char *)Alloca(GZ_BUFFER_SIZE))) {
    if ((FI = fopen(Src, "rb"))) {
      if ((FO = gzopen(Dest, "wb"))) {
        while ((Ret) && (Rd = fread(Buf, 1, GZ_BUFFER_SIZE, FI))) {
          if (gzwrite(FO, Buf, Rd) != Rd)
            Ret = GZ_Error(FO);
        }
        if (ferror(FI)) Ret = PrintDosErr();
        gzclose(FO);
      } else Ret = GZ_Error(FO);
      fclose(FI);
    } else Ret = PrintDosErr();
  } else Ret = FALSE;

  return Ret;
}


/**** Show a gzip error ****/

static short GZ_Error(gzFile *FH)
{
  int           err;

  return CatErr(MSG_ERR_GZIP_ERROR, gzerror(FH, &err));
}
