
/*************************************************
****               VEGA - Z V1.1              ****
****         Interface to Unix compress       ****
**** Copyright 1996-2000, Alessandro Pedretti ****
*************************************************/


#include <stdio.h>
#include <string.h>

#include "globstr.h"
#include "../Z32/z32.h"

#define  FALSE                     0
#define  TRUE                      1
#define  Z_BUFFER_SIZE         65536


/**** Local prototypes ****/

/* Some VEGA definitions are incompatible with the standard windows.h */
/* file and thus can't be used in the following prototypes            */

void         *Alloca(unsigned int);
short        CatErr(unsigned int, ...);
short        Z_DepackFile(char *, char *);
static short Z_Error(ZSTREAM *FH);
short        Z_PackFile(char *, char *);
short        PrintDosErr(void);


/**** Decompress a file into another ****/

short Z_DepackFile(char *Src, char *Dest)
{
  char          *Buf;
  FILE          *FO;
  int           Rd;
  ZSTREAM       *FI;

  short        Ret = TRUE;

  if ((Buf = (char *)Alloca(Z_BUFFER_SIZE))) {
    if ((FI = zopen(Src, "r", 0))) {
      if ((FO = fopen(Dest, "wb"))) {
        while((Ret) && (Rd = zread(FI, Buf, Z_BUFFER_SIZE))) {
          if (Rd == -1)
            Ret = Z_Error(FI);
          else if (fwrite(Buf, Rd, 1, FO) != 1)
            Ret = PrintDosErr();
        }
        fclose(FO);
      } else Ret = PrintDosErr();
      zclose(FI);
    } else Ret = Z_Error(FI);
  } else Ret = FALSE;

  return Ret;
}


/**** Compress a file into another ****/

short Z_PackFile(char *Src, char *Dest)
{
  char          *Buf;
  FILE          *FI;
  int           Rd;
  ZSTREAM       *FO;

  short        Ret = TRUE;

  if ((Buf = (char *)Alloca(Z_BUFFER_SIZE))) {
    if ((FI = fopen(Src, "rb"))) {
      if ((FO = zopen(Dest, "w", 0))) {
        while ((Ret) && (Rd = fread(Buf, 1, Z_BUFFER_SIZE, FI))) {
          if (zwrite(FO, Buf, Rd) != Rd)
            Ret = Z_Error(FO);
        }
        if (ferror(FI)) Ret = PrintDosErr();
        zclose(FO);
      } else Ret = Z_Error(FO);
      fclose(FI);
    } else Ret = PrintDosErr();
  } else Ret = FALSE;

  return Ret;
}


/**** Show a Z error ****/

static short Z_Error(ZSTREAM *FH)
{
  int           err;

  return CatErr(MSG_ERR_Z32_ERROR, zerror(FH, &err));
}
